/********************************************************************/
/*                                                                  */
/*   Program Name :  setdcoffset                                    */
/*                                                                  */
/*      Version:  1.00  2007-02-16                                  */
/*                1.10  2007-02-23  revised to use common library   */
/*                                                                  */
/*      Copyright (c) 2007 T.Kondo/NICT All Right Reserved          */
/*                                                                  */
/********************************************************************/
/*
  set DC offset of K5/VSSP32 sampler unit. As for K5/VSSP, all mode is regarded
  as "STAT" mode

         by T.Kondo 2007-02-15   start development
                    2007-02-16   add RESET mode
                    2007-02-23   revised to use common library

    How to use
    
    (1) just set DC offset
    
      setdcoffset dc1 dc2 dc3 dc4
      
           where  dc1  -- DC offset for ch 1
                  dc2  -- DC offset for ch 2
                  dc3  -- DC offset for ch 3
                  dc4  -- DC offset for ch 4
                  
    (2) reset or set DC offset adequate for borad running test or 
      observation
      
      setdcoffset TEST|AUTO|OBS|RESET
      
          where TEST -- set DC offset so as to keep data 
                        all 0 in case of no signal case
                       (DC offset -6 will be set)
                AUTO -- set DC offset adequate for observation
                OBS  -- same as AUTO
                STAT -- display statistics only
                RESET - set DC offset all 0
      
*/
#define PROGRAM_VERSION "Ver. 2007-02-23"

#include "chead.h"      // common header for IP-VLBI utilities
#include "libipvlbi_board.h"  // IP-VLBITv[{[hp֐Q(by T.Kondo)

#define N12(a,b) (a+b*4)   //  macro to convert 2D array pointer to 1D array pointer
                                    //  for array(4,256)

/* set global variables */
char dev[] = DEV_FILE;  // IP-VLBI BOARD control variable
int fd_in;                 // IP-VLBI BOARD control variable


int get_signal_statistics(double *ave, double *sigma, int *naveall, int *ihist, int *getdc)
{
    int rv;
    //int ihist[4*256];
    int w1,w2,w3,mm;
    int n;
    double d1;
    //double sum[4],ave[4],sum2[4],ave2[4],sigma[4];
    double sum[4],sum2[4],ave2[4];
    int nave;
    char cbuf[20];
#ifdef _VSSP32
    struct tdsio tdsio;   // for setting DC_OFFSET or getting DC_OFFSET
#endif
    
#ifdef _VSSP32
    int headbit=256;  // default header bit number  for K5/VSSP32
#else
    int headbit=64;  // header bit number  for K5/VSSP
#endif
    int allbyte;     // TvOoCg
    int bitsec;       // Pb̃rbg@iwb_[܂ށj
    int bytesec;       // Pb̃oCg@iwb_[܂ށj
    int c,k;
    int nkai;
    int err, adjusted;
    //int dcoffset[4];
    //unsigned int sampling1, freq, width, stat;
    int sampling1, freq, width, stat, filter0;
    unsigned int ref, pps, rhz, pll;
    int ch, pos;
    vssp_param_t sp;
    u_int8_t sfreq_t, sbit_t, numch_t;
    u_int8_t sampling;
    //u_int32_t buf[BUFSIZ * 2];
    u_int32_t buf[1024 * 2];
    double telapsed;           // to measure processing time
    time_t tstart, tfinish;    //           "
    //printf("BUFSIZ   %d\n",BUFSIZ);   // BUFSIZ: 1024 for K5/VSSP   8192 for K5/VSSP32
    //int xdata[BUFSIZ * 16];  // f[^̃f[^[p
    int xdata[1024 * 16];  // f[^̃f[^[p
    
    int nbyte = sizeof(buf);   // nbyte = 8192

    //printf(" span = %d\n",span);    // debug
    //printf(" sfreq= %d\n",sfreq);   // debug

loopmain:

    sp.sfreq=1;
    sp.adbit = 8;
    sp.filter=0;  // K5/VSSP32 parameter
    sp.numch=4;

    /* sampler open */
    fd_in = open(dev, O_RDONLY);
    //fd_in = open("/dev/tds0", O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    

#ifdef _VSSP32
    if((rv=ioctl(fd_in,TDSIO_INIT)) == -1) {
        perror("ioctl() INIT failed ");
        goto end;
    }
#endif
    /* board status check */
    if((rv=check_vssp(fd_in, &stat, 1)) == -1) {
        perror("ioctl() status get failed ");
        goto end;
    }

    /* sampling parameters set */
    //if((rv=set_vssp_sparam(fd_in,sfreq,adbit,numch,filter,1))==-1){
    if((rv=set_vssp_sparam(fd_in, &sp,1))==-1){
        printf("sampling set failed \n");
        goto end;
    }

    if (sp.sfreq >= 40) {
       bitsec=sp.sfreq*1000 + headbit ;
    } else { 
       bitsec=sp.sfreq*1000*1000 + headbit ;
    }

    bitsec=bitsec*sp.adbit*sp.numch;   // bit number per second

    bytesec=bitsec/8;
    
    /* get sampling parameters from a board */
    if ((rv=get_vssp_sparam(fd_in,&sp,1))== -1) {
        perror("ioctl() get sampling parameter failed ");
        goto end;
    } else {
//        printf("sampling setting is %d kHz %d BIT %d CH",
//                  sp.freqkhz,sp.adbit,sp.numch);
#ifdef _VSSP32
//        printf(" FILTER %d MHz\n",sp.filter);
#else
//        printf("\n");
#endif
    }   

    rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);   // {[hFIFÕNA[
    if (rv == -1) {
        perror("ioctl() FIFO clear failed ");
        goto end;
    } else {
        printf("ioctl() FIFO clear succeed\n");
    }

    rv = ioctl(fd_in, TDSIO_SAMPLING_START);
    if (rv == -1) {
        perror("ioctl() start sampling failed ");
              printf("Probably time was not set for the IP-VLBI board.\n");
              printf("Please set time using timesettk and try again.\n");
        goto end;
    } else {
        printf("ioctl() start sampling succeed\n");
    }
    

    /* array data initialize */
    for(w1=0; w1<4; w1++){
       for(k=0; k<256; k++){
          ihist[N12(w1,k)]=0;
       }
       sum[w1]=0.0;
       ave[w1]=0.0;
       sum2[w1]=0.0;
       ave2[w1]=0.0;
       sigma[w1]=0.0;
    }
    nave=0;
    nkai=2;
    rv = read(fd_in, buf, 32); // to avoid 1st header 
    for(n=0; n<nkai; n++){
       rv = read(fd_in, buf, nbyte);
       if (rv == -1) {
          printf("read() failed   ");
          ioctl(fd_in, TDSIO_GET_ERROR, &err); printf("Error code = %d\n",err);  // debug for VSSP32
          tfinish= time(NULL);
          telapsed = difftime(tfinish,tstart);
          printf("Error occurred after %f minutes sampling.\n",telapsed/60.0);
          goto end;
       }

       parseADdata(buf, rv, sp.adbit, xdata, &pos);
       
       /* calc average and standard deviation of signals */

       w3=0;  // channel index
       for(k=0; k<pos; k++){
          w1=xdata[k];
          ihist[N12(w3,w1)]+=1;
          d1=(double)w1- 255.0/2.0;  // assuming 8 bit AD
          sum[w3]+=d1;  // for average calc
          sum2[w3]+=(d1*d1);  // for variance calc
          w3++;
          if( w3 == 4 ) w3 =0; 
          if( sp.numch == 1 ) w3 =0; 
       }
       nave+=pos/sp.numch;
       for(k=0; k<sp.numch; k++) {
           ave[k]=sum[k]/(double)nave;
           ave2[k]=sum2[k]/(double)nave;
           sigma[k]=sqrt(ave2[k]-ave[k]*ave[k]);  // standard deviation
       }
    }


    rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
    if (rv == -1) {
        perror("ioctl() stop sampling failed ");
        goto end;
    }
    else {
        printf("ioctl() stop sampling succeed\n");
    }
#ifdef _VSSP32
    /* DC offset get */  
    for(k=0; k<4; k++){
       tdsio.key=k;
       if ((rv=ioctl(fd_in, TDSIO_GET_DCOFFSET, &tdsio)) == -1) {
          printf("ioctl() get DCoffset failed\n");
       }
       getdc[k]= tdsio.value;
       if(tdsio.value > 127) getdc[k]= tdsio.value-256;
    }
#endif

 end:
    close(fd_in);
    *naveall=nave;
    return rv;
}



int set_offset(int adjust, int *setdc, int *getdc)
{
   /*
       adjust ----  mode to set DC offset
                    0 :  nothing to do
                    1 :  set DC offset
                    2 :  reset DC offset
   */
    int rv;
    int k,mm;
    int adjusted;
    int dcoffset[4];
    int stat;
#ifdef _VSSP32
    struct tdsio tdsio;   // for setting DC_OFFSET or getting DC_OFFSET
#endif
    

    /* check board  */
#ifndef _VSSP32
    printf(" Nothing to do\n");
    return -1;
#endif
    if(adjust == 0){
       printf(" Nothing to do\n");
       return -1;
    }
    

    adjusted=0;

loopmain:

    /* sampler open */
    fd_in = open(dev, O_RDONLY);
    if (fd_in == -1) {
        perror("device open() error ");
        return -1;
    }
    

#ifdef _VSSP32
    if((rv=ioctl(fd_in,TDSIO_INIT)) == -1) {
        perror("ioctl() INIT failed ");
        goto end;
    }
#endif
    /* board status check */
    if((rv=check_vssp(fd_in, &stat, 1)) == -1) {
        perror("ioctl() status get failed ");
        goto end;
    }


#ifdef _VSSP32
    /* DC offset reset */
    if(adjust==2){
        for(k=0; k<4; k++){
           tdsio.key=k;
           tdsio.value=0;
           if ((rv=ioctl(fd_in, TDSIO_SET_DCOFFSET, &tdsio)) == -1) {
              printf("ioctl() set DCoffset failed\n");
           }
        }
    }
 
    /* DC offset get */  
    for(k=0; k<4; k++){
       tdsio.key=k;
       if ((rv=ioctl(fd_in, TDSIO_GET_DCOFFSET, &tdsio)) == -1) {
          printf("ioctl() get DCoffset failed\n");
       }
       getdc[k]= tdsio.value;
       if(tdsio.value > 127) getdc[k]= tdsio.value-256;
    }
    
    
#endif

#ifdef _VSSP32
    
    printf("  Adjusting DC offset......");
    if((rv=ioctl(fd_in,TDSIO_INIT)) == -1) {
           perror("ioctl() INIT failed ");
           goto end;
    }
    for(mm=0; mm<4; mm++){
        /* DC offset set */  
          tdsio.key=mm;
          tdsio.value = setdc[mm];
          printf("  CH %d = %d",mm+1, setdc[mm]);
          if ((rv=ioctl(fd_in, TDSIO_SET_DCOFFSET, &tdsio)) == -1) {
              printf("ioctl() set DCoffset failed\n");
          }
    }
    printf("\n");
    printf("  Adjusting Finished ......\n");
    adjust=0;
    adjusted=1;
    /* DC offset get */  
    for(k=0; k<4; k++){
       tdsio.key=k;
       if ((rv=ioctl(fd_in, TDSIO_GET_DCOFFSET, &tdsio)) == -1) {
          printf("ioctl() get DCoffset failed\n");
       }
       getdc[k]= tdsio.value;
       if(tdsio.value > 127) getdc[k]= tdsio.value-256;
    }
#endif
    
 end:

    close(fd_in);

    return rv;
}


int main(int argc, char **argv)
{
    //char dev[] = "/dev/tds0";
    char outfile[] = "./tds.data";
    char cbuf[100];
    char* dflt_logfile="dcoffset.log";
    char logfile[80];
    int logmode;
    int rv;
    int setdc[4],getdc[4];
    int dc1,dc2,dc3,dc4;
    int adjust;
    int n, mode;
    int naveall;
    int ihist[4*256];
    int offset;
    double ave[4],sigma[4];
    logval_t lg;

    logmode=0;
    if(argc <= 1 ){
howtouse:
        printf("Setdcoffset  (%s)\n",PROGRAM_VERSION);
        printf("  sets DC offset of K5/VSSP32 sampler. As for K5/VSSP, all mode\n");
        printf("   is regarded as STAT (statistics display) mode \n");
        printf("Usage: \n");
        printf("    setdcoffset dc1 dc2 dc3 dc4 [logfile]\n");
        printf("         where  dc1 ---- DC offset for ch1 (-128 to +128)\n");
        printf("                dc2 ---- DC offset for ch2 (-128 to +128)\n");
        printf("                dc3 ---- DC offset for ch3 (-128 to +128)\n");
        printf("                dc4 ---- DC offset for ch4 (-128 to +128)\n");
        printf("                logfile -- log file name\n");
        printf("                          default is dcoffset.log\n");
        printf("                          if default logfile is used, it is overwritten.\n");
        printf("                          But if logfile is set by \"logfile\" parameter,\n");
        printf("                          log will be appended to logfile.\n");
        printf("   or\n");
        printf("    setdcoffset TEST|AUTO|OBS|STAT|RESET [logfile]\n");
        printf("         where  TEST  -- set DC offset adequately for \n");
        printf("                         board test\n");
        printf("                AUTO  -- set DC offset for observation\n");
        printf("                OBS   -- same as AUTO\n");
        printf("                STAT  -- display statistics only\n");
        printf("                RESET -- set DC offset all 0\n");
        printf("                logfile -- see above\n");
        printf("\n");
        exit(0);
    }
    for(n=0; n<4; n++) setdc[n]=0;
    
    mode=0;        // 0: set each DC offset given by operator
                   // 1: TEST
                   // 2: AUTO or OBS mode
                   // 3: RESET mode
    if(argc >1){
       if(strncmp("TEST",*(argv + 1 ),4)==0) mode=1;
       if(strncmp("test",*(argv + 1 ),4)==0) mode=1;
       if(strncmp("AUTO",*(argv + 1 ),4)==0) mode=2;
       if(strncmp("auto",*(argv + 1 ),4)==0) mode=2;
       if(strncmp("OBS",*(argv + 1 ),3)==0 ) mode=3;
       if(strncmp("obs",*(argv + 1 ),3)==0 ) mode=3;
       if(strncmp("STAT",*(argv + 1 ),4)==0 ) mode=4;
       if(strncmp("stat",*(argv + 1 ),4)==0 ) mode=4;
       if(strncmp("RESET",*(argv + 1 ),5)==0) mode=5;
       if(strncmp("reset",*(argv + 1 ),5)==0) mode=5;
    }
    if(mode==0 && argc <=4) goto howtouse;
    n=2;
    if(mode ==0){
       if(argc >1) setdc[0]=atoi(*(argv + 1 ));
       if(argc >2) setdc[1]=atoi(*(argv + 2 ));
       if(argc >3) setdc[2]=atoi(*(argv + 3 ));
       if(argc >4) setdc[3]=atoi(*(argv + 4 ));
       n=5;
    }
    
    if(argc> n){
          strcpy(logfile,*(argv+n));
          logmode=1; // log is appended to existed log file
    } else {
          strcpy(logfile,dflt_logfile);
    }
    /* log info set if necessary */
    strcpy(lg.logfile,logfile);
    lg.hostname[0]='\0';
    lg.logdir[0]='\0';
    lg.logfiletype=0;    //-1: no log   0: log out
    if(logmode==0){
       sprintf(cbuf,"rm -f %s\0",logfile);
       system(cbuf);    // delete logfile
    }
    
#ifndef _VSSP32
    mode=4;  // compulsory set mode
#endif
    
    switch(mode){
       case 0: goto p_start2;
       case 1: goto p_start1;
       case 2: goto p_start1;
       case 3: goto p_start1;
       case 4: goto statistics;
       case 5: goto p_start2;
    }
    
p_start1:
    
    /* 1st reset DC offset */
    for(n=0; n<4; n++) setdc[n]=0;
    adjust=1;
    rv=set_offset(adjust,setdc,getdc);
    
    /* get signal statistics */
    printf("  get signal statistics......\n");
    rv=get_signal_statistics(ave, sigma, &naveall, ihist, getdc);
    
    /* print DC offset info */
    printf(" DC OFFSET  ");
    for(n=0; n<4; n++) printf("%6.1f/256",ave[n]);
    printf("\n");
    
    /* print standard deviation info */
    printf(" ONE SIGMA  ");
    for(n=0; n<4; n++) printf("%6.1f/256",sigma[n]);
    printf("\n");
    
    if(mode ==1) {
       for(n=0; n<4; n++){
          if(sigma[n] < 2.0) { // this may be no signal
             // to make sure data all 0
             if(ave[n] > 0){
                offset=-(int)ave[n]-6;
             } else {
                offset=-6;
             }
             setdc[n]=offset;
          } else { // some signal detected
             setdc[n]=-(int)ave[n];
          }
      }
   } else if (mode==2 || mode==3){
       for(n=0; n<4; n++){
          setdc[n]=-(int)ave[n];
      }
   }
      
           
p_start2:
           
    
    adjust=1;

    printf("*** SETDCOFFSET (set DC offset for K5/VSSP32) (%s) ** *\n",PROGRAM_VERSION);

    rv=set_offset(adjust,setdc,getdc);

    printf("  set offset  CH1 %d   CH2 %d   CH3 %d   CH4 %d\n",
                 getdc[0],getdc[1],getdc[2],getdc[3]);


statistics:
    /* get signal statistics */
    printf("  get signal statistics......\n");
    rv=get_signal_statistics(ave, sigma, &naveall, ihist, getdc);
    
    /* log out */
#ifdef _VSSP32
    sprintf(cbuf,"setdcoffset:  \0");
    switch(mode){
       case 0: 
            sprintf(cbuf,"setdcoffset: set %d  %d  %d  %d\0",
                 setdc[0],setdc[1],setdc[2],setdc[3]);
            break;
       case 1: 
            sprintf(cbuf,"setdcoffset: set TEST mode\0");
            break;
       case 2: 
            sprintf(cbuf,"setdcoffset: set AUTO mode\0");
            break;
       case 3: 
            sprintf(cbuf,"setdcoffset: set OBS mode\0");
            break;
       case 4: 
            sprintf(cbuf,"setdcoffset: set RESET mode\0");
    }
    printf("%s\n",cbuf);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
    sprintf(cbuf,"setdcoffset: VSSP32 OFFSET CH1 = %d  CH2 = %d  CH3 = %d  CH4 = %d\0",
                 getdc[0],getdc[1],getdc[2],getdc[3]);
    printf("%s\n",cbuf);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
#endif
    sprintf(cbuf,"setdcoffset:                       Signal Statistics\0");
    printf("%s\n",cbuf);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
    sprintf(cbuf,"setdcoffset:               CH1        CH2        CH3        CH4\0");
    printf("%s\n",cbuf);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
    /* print DC offset info */
    sprintf(cbuf,"setdcoffset: average  %6.1f/256 %6.1f/256 %6.1f/256 %6.1f/256\0",
                           ave[0],ave[1],ave[2],ave[3]);
    printf("%s\n",cbuf);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
       
    /* print standard deviation info */
    sprintf(cbuf,"setdcoffset: std dev  %6.1f/256 %6.1f/256 %6.1f/256 %6.1f/256\0",
                           sigma[0],sigma[1],sigma[2],sigma[3]);
    printf("%s\n",cbuf);
    LogWrite4(&lg, 0, 0.0, 0, cbuf);
    
    if(lg.logfiletype==0)  printf(" logfile = %s\n",logfile); 

    return 0;
}
